package org.openxri.admin.resources;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Stroke;

import org.apache.wicket.Application;
import org.apache.wicket.markup.html.image.resource.RenderedDynamicImageResource;
import org.apache.wicket.markup.html.link.ImageMap;
import org.openxri.admin.webapplication.OpenXRIAdminApplication;
import org.openxri.exceptions.StoreException;
import org.openxri.store.Store;
import org.openxri.store.SubSegment;

public class SubSegmentImageResource extends RenderedDynamicImageResource {

	private static final long serialVersionUID = -6682154691595772165L;

	protected static final int IMAGE_WIDTH = 800;
	protected static final int IMAGE_HEIGHT = 200;
	protected static final int IMAGE_PADDING_X = 0;
	protected static final int IMAGE_PADDING_Y = 0;
	protected static final int LEFT = 0 + IMAGE_PADDING_X;
	protected static final int TOP = 0 + IMAGE_PADDING_Y;
	protected static final int RIGHT = IMAGE_WIDTH - IMAGE_PADDING_X;
	protected static final int BOTTOM = IMAGE_HEIGHT - IMAGE_PADDING_Y;
	protected static final int WIDTH = RIGHT - LEFT;
	protected static final int HEIGHT = BOTTOM - TOP;
	protected static final int CENTER_X = IMAGE_WIDTH / 2;
	protected static final int CENTER_Y = IMAGE_HEIGHT / 2;
	protected static final int CENTER_AUTHORITY_WIDTH = 20;
	protected static final int CENTER_AUTHORITY_HEIGHT = 20;
	protected static final int AUTHORITY_WIDTH = 20;
	protected static final int AUTHORITY_HEIGHT = 20;

	protected static final Color AUTHORITY_COLOR = Color.DARK_GRAY;
	protected static final Color LINE_COLOR = new Color(0, 170, 255);
	protected static final Stroke LINE_STROKE = new BasicStroke(3);
	protected static final Color TEXT_COLOR = Color.WHITE;
	protected static final Font TEXT_FONT = new Font("Sans-serif", Font.PLAIN, 10);

	protected boolean parentAuthorityNull;
	protected String name;
	protected boolean authorityNull;

	public SubSegmentImageResource(SubSegment subSegment) throws StoreException {

		super(IMAGE_WIDTH, IMAGE_HEIGHT, "png");

		final Store openXRIStore = ((OpenXRIAdminApplication) Application.get()).getOpenXRIStore();

		this.name = subSegment.getName();
		this.parentAuthorityNull = openXRIStore.getSubSegmentParentAuthority(subSegment) == null;
		this.authorityNull = openXRIStore.getSubSegmentAuthority(subSegment) == null;
	}

	@Override
	protected boolean render(Graphics2D graphics) {

		graphics.setFont(TEXT_FONT);

		renderSubSegment(graphics);

		return(true);
	}

	protected void renderSubSegment(Graphics2D graphics) {

		int y1, y2;
		
		y1 = TOP + HEIGHT / 4;
		y2 = TOP + HEIGHT * 3 / 4;
		
		graphics.setColor(LINE_COLOR);
		graphics.setStroke(LINE_STROKE);
		graphics.drawLine(CENTER_X, y1, CENTER_X, y2);
		
		graphics.setColor(AUTHORITY_COLOR);
		graphics.setBackground(AUTHORITY_COLOR);
		if (this.parentAuthorityNull) {

			graphics.drawOval(CENTER_X - CENTER_AUTHORITY_WIDTH / 2, y1 - CENTER_AUTHORITY_HEIGHT / 2, CENTER_AUTHORITY_WIDTH, CENTER_AUTHORITY_HEIGHT);
		} else {

			graphics.fillOval(CENTER_X - CENTER_AUTHORITY_WIDTH / 2, y1 - CENTER_AUTHORITY_HEIGHT / 2, CENTER_AUTHORITY_WIDTH, CENTER_AUTHORITY_HEIGHT);
		}
		
		graphics.setColor(AUTHORITY_COLOR);
		graphics.setBackground(AUTHORITY_COLOR);
		if (this.authorityNull) {

			graphics.drawOval(CENTER_X - CENTER_AUTHORITY_WIDTH / 2, y2 - CENTER_AUTHORITY_HEIGHT / 2, CENTER_AUTHORITY_WIDTH, CENTER_AUTHORITY_HEIGHT);
		} else {
			
			graphics.fillOval(CENTER_X - CENTER_AUTHORITY_WIDTH / 2, y2 - CENTER_AUTHORITY_HEIGHT / 2, CENTER_AUTHORITY_WIDTH, CENTER_AUTHORITY_HEIGHT);
		}

		int posX = CENTER_X;
		int posY = CENTER_Y;

		FontMetrics fontMetrics = graphics.getFontMetrics(TEXT_FONT);
		double textWidth = fontMetrics.getStringBounds(this.name, graphics).getWidth();
		double textHeight = fontMetrics.getStringBounds(this.name, graphics).getHeight();
		
		graphics.setColor(TEXT_COLOR);
		graphics.drawString(this.name, posX - (int) (textWidth/2), posY - (int) (textHeight/2) + fontMetrics.getAscent());
	}
		

	public ImageMap createImageMap(String id) {

		ImageMap map = new ImageMap(id);

		return(map);
	}
}
