package org.openxri.admin.webpages.home;

import org.apache.wicket.Application;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.pages.RedirectPage;
import org.apache.wicket.model.CompoundPropertyModel;
import org.openxri.XRI;
import org.openxri.admin.webapplication.OpenXRIAdminApplication;
import org.openxri.admin.webpages.BasePage;
import org.openxri.config.ServerConfig;

public class Index extends BasePage {

	private static final long serialVersionUID = -597481320422867019L;

	private final String _xrifail = this.getString("xrifail");

	public Index() {

		this.setTitle(this.getString("title"));

		ServerConfig openXRIConfig = ((OpenXRIAdminApplication) Application.get()).getOpenXRIServerConfig();

		// create and add components

		String serverUrl = openXRIConfig.getHostName() + ":" + openXRIConfig.getHostPort() + openXRIConfig.getServletPath();
		this.add(new Label("serverUrl", serverUrl));
		this.add(new QuickResolutionForm("quickResolutionForm"));
		this.add(new QuickRetrievalForm("quickRetrievalForm"));
	}

	private class QuickResolutionForm extends Form {

		private static final long serialVersionUID = -1539881618410408448L;

		private String xri;

		public QuickResolutionForm(String id) {

			super(id);
			this.setModel(new CompoundPropertyModel(this));

			// create and add components

			this.add(new TextField("xri"));
		}

		public String getXri() {
			return (this.xri);
		}
		public void setXri(String xri) {
			this.xri = xri;
		}

		@Override
		protected void onSubmit() {

			try {

				new XRI(this.xri);
			} catch (Exception ex) {

				Index.this.error(Index.this._xrifail);
				return;
			}

			String hxri = ((OpenXRIAdminApplication) this.getApplication()).getProperties().getProperty("hxri-prefix") + this.xri;
			this.setResponsePage(new RedirectPage(hxri));
		}
	}

	private class QuickRetrievalForm extends Form {

		private static final long serialVersionUID = -7274158620417007644L;

		private String xri;

		public QuickRetrievalForm(String id) {

			super(id);
			this.setModel(new CompoundPropertyModel(this));

			// create and add components

			this.add(new TextField("xri"));
		}

		public String getXri() {
			return (this.xri);
		}
		public void setXri(String xri) {
			this.xri = xri;
		}

		@Override
		protected void onSubmit() {

			try {

				new XRI(this.xri);
			} catch (Exception ex) {

				Index.this.error(Index.this._xrifail);
				return;
			}

			String hxri = ((OpenXRIAdminApplication) this.getApplication()).getProperties().getProperty("hxri-prefix") + this.xri;
			this.setResponsePage(new RedirectPage(hxri + "?_xrd_r=application/xrds+xml"));
		}
	}
}
