package org.openxri.admin.webpages.store;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.link.PageLink;
import org.apache.wicket.model.CompoundPropertyModel;
import org.openxri.admin.webapplication.OpenXRIAdminApplication;
import org.openxri.admin.webpages.BasePage;
import org.openxri.exceptions.StoreException;
import org.openxri.store.Store;
import org.openxri.xml.XRD;

public class CreateRootNamespace extends BasePage {

	private static final long serialVersionUID = 5529090374026849299L;

	private static final Log log = LogFactory.getLog(CreateRootNamespace.class);

	private final String _storefail = this.getString("storefail");
	private final String _success = this.getString("success");

	public CreateRootNamespace() {

		this.setTitle(this.getString("title"));

		// create and add components

		this.add(new CreateRootNamespaceForm("createRootNamespaceForm"));
		this.add(new PageLink("backLink", Index.class));
	}

	private class CreateRootNamespaceForm extends Form {

		private static final long serialVersionUID = -1852614980935432265L;

		private String namespace;

		public CreateRootNamespaceForm(String id) {

			super(id);
			this.setModel(new CompoundPropertyModel(this));

			// create and add components

			this.add(new TextField("namespace").setRequired(true));
		}

		public String getNamespace() {
			return (this.namespace);
		}

		public void setNamespace(String namespace) {
			this.namespace = namespace;
		}

		@Override
		protected void onSubmit() {

			final Store openXRIStore = ((OpenXRIAdminApplication) this.getApplication()).getOpenXRIStore();

			try {

				openXRIStore.createRootSubSegment(this.namespace, new XRD());
			} catch (StoreException ex) {

				CreateRootNamespace.log.error(ex);
				CreateRootNamespace.this.error(CreateRootNamespace.this._storefail + ex.getLocalizedMessage());
				return;
			}

			CreateRootNamespace.this.info(CreateRootNamespace.this._success);
		}
	}
}
